import { system, world, BlockPermutation, Direction, EntityDamageCause, EquipmentSlot, GameMode, ItemStack } from "@minecraft/server";
import * as defaultWorldArrays from "./default_world_arrays.js"
import * as worldUtilities from "./world_utilities.js"

world.beforeEvents.itemUseOn.subscribe(({source:player,block,itemStack}) => {
	const blockAbove = block.dimension.getBlock(block.location).above(1);
	const id = itemStack.typeId;
	const array = defaultWorldArrays;
	if (array.addonNamespaces.some(prefix=>id.startsWith(prefix))) {
		if (itemStack.hasTag("minecraft:is_shovel")) {
			if (array.isShovelTagUseOnBlocksToGrassPath.includes(block.typeId)) {
				if (!blockAbove.isAir) return;
				system.run(() => {
					worldUtilities.interactToolUse(block,itemStack,player,array.isShovelTagUseOnBlocks);
					return block.setType("minecraft:grass_path");
				});
			}
			if (array.isShovelTagUseOnBlocksToExtinguish.includes(block.typeId)) {
				const extinguished = block.permutation.getState("extinguished");
				if (extinguished==true) return;
				system.run(() => {
					worldUtilities.interactToolUse(block,itemStack,player,array.isShovelTagUseOnBlocks);
					return block.setPermutation(block.permutation.withState("extinguished",true));
				});
			}
		}
    }
});
world.afterEvents.playerInteractWithBlock.subscribe(({block,itemStack,player}) => {
	const id = itemStack.typeId;
	const array = defaultWorldArrays;
	if (array.addonNamespaces.some(prefix=>id.startsWith(prefix))) {
		if (itemStack.hasTag("minecraft:is_axe")) return worldUtilities.interactToolUse(block,itemStack,player,array.isAxeTagInteractBlocks);
		if (itemStack.hasTag("minecraft:is_hoe")) return worldUtilities.interactToolUse(block,itemStack,player,array.isHoeTagInteractBlocks);
		if (itemStack.hasTag("minecraft:is_shovel")) return worldUtilities.interactToolUse(block,itemStack,player,array.isShovelTagInteractBlocks);
	}
});

function applyDurability(player) {
    if (player.getGameMode()==GameMode.creative) return;
		const inv = player.getComponent("inventory")?.container;
    const slot = inv?.getSlot(player.selectedSlotIndex);
    const item = slot?.getItem();
    if (!item || !item.hasTag("mvt:durability")) return;
    const durability = item.getComponent("durability");
    if (!durability) return;
    const enchantments = item.getComponent("enchantments");
    let unbreakingLevel = 0;
    if (enchantments) {
        const unbreaking = enchantments.getEnchantment(EnchantmentType.unbreaking);
        if (unbreaking) unbreakingLevel = unbreaking.level;
    }
    
    // Unbreaking chance logic
    const chance = 1 / (unbreakingLevel + 1);
    if (Math.random() < chance) {
        durability.damage += 1;
    }
    
    if (durability.damage >= durability.maxDurability) {
        player.playSound("random.break");
        slot.setItem(undefined);
    } else {
        slot.setItem(item);
    }
}

world.afterEvents.playerBreakBlock.subscribe(({ player }) => {
    if (player) applyDurability(player);
});